# compile
cd ../rogueviz
make hyper.o rogueviz.o savepng-loc.o
cd ../hyperbolic-embedder
make main
cd ..
make bin/dhrg
make bin/dhrg-64
cd simulated

# make directories
mkdir -p data times route groute output embout

# embed with default temperature
function embed {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/emb-$size-$proc$att ../hyperbolic-embedder/embedder --generate data/simg-$size-$proc$att -n $size -seed $2$att -embed data/sime-$size-$proc$att >embout/$size-$proc$att
}

# embed with T = .7
function embed7 {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/emb-$size-$proc$att ../hyperbolic-embedder/embedder --generate data/simg-$size-$proc$att -n $size -T .7 -seed $id$att -embed data/sime-$size-$proc$att >embout/$size-$proc$att
}

# compute the ground truths
function gtt {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/dhrg-$size-$proc$att ../bin/dhrg -nogui -7 -gtt $size-$proc$att > output/$size-$proc$att
}

# test the routing on G710
function route {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/route-$size-$1$att ../bin/dhrg -nogui -7 -routing $size-$1$att > route/$size-$1$att
}

# test the routing on G710, using dhrg-64 (large distances)
function route64 {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/route-$size-$1$att ../bin/dhrg-64 -nogui -7 -routing $size-$1$att > route/$size-$1$att
}

# test the routing on G711
function groute {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/groute-$size-$1$att ../bin/dhrg -nogui -routing $size-$1$att > groute/$size-$1$att
}

# test the routing on G711, using dhrg-64 (large distances)
function groute64 {
proc=$1
id=$2
size=$3
att=$4
/usr/bin/time -p -o times/groute-$size-$1$att ../bin/dhrg-64 -nogui -routing $size-$1$att > groute/$size-$1$att
}

function groute64-new {
proc=$1
id=$2
size=$3
att=$4
if [ ! -e groute/$size-$1$att ]; then /usr/bin/time -p -o times/groute-$size-$1$att ../bin/dhrg-64 -nogui -routing $size-$1$att > groute/$size-$1$att; fi
}

# for quick testing, only 10x8; we created 125x8

function repiter {
for att in {00..09} # {00..19} {20..124}
do
$1 $2 $3 $4 $att
done
}

function repproc {
echo `date` performing operation: $1 $2 >> simulations-log.txt
repiter $1 a 1 $2 &
repiter $1 b 2 $2 &
repiter $1 c 3 $2 &
repiter $1 d 4 $2 &
repiter $1 e 5 $2 &
repiter $1 f 6 $2 &
repiter $1 g 7 $2 &
repiter $1 h 8 $2 &
wait
wait
wait
wait
wait
wait
wait
wait
echo `date` finished operation: $1 $2 >> simulations-log.txt
}

# for quick testing, only small graphs
for size in 100 # 200 300 500 1000 2000 3000 4000 5000 10000 15000
do
repproc embed $size
repproc gtt $size
repproc route64 $size
repproc groute64 $size
done
#route groute is enough for smaller graphs

# for quick testing, only small graphs
for size in 270 # 470 970 1970 2970 3970 4970 9970 14970
do
repproc embed7 $size
repproc gtt $size
repproc route64 $size
repproc groute64 $size
done
#route groute is enough for smaller graphs

mkdir results
for y in route groute output; do for x in `ls $y/ | sort -g`; do tail $y/$x | grep RES | sed s/RES/$x/; done | sh numberer.sh > results/$y.csv; done
