// general utilities
#include <sys/time.h>

namespace dhrg {

typedef long long ll;

long long getVa() {
  struct timeval tval;
  gettimeofday(&tval, NULL);
  return tval.tv_sec * 1000000 + tval.tv_usec;
  }

string timeshow(long long t) {
  if(t < 1000000) return its(t) + " µs";
  else if (t < 1000000000) return its((t+999)/1000) + " ms";
  else return its((t+999999)/1000000) + " s";
  }

int indent;

void doindent() {
  for(int i=0; i<indent; i++) printf(" ");
  }

#ifndef XPRINTF
#ifdef __GNUC__
__attribute__((__format__ (__printf__, 1, 2)))
#endif
inline void iprintf(const char *fmt, ...) {
    doindent();
    va_list ap;
    va_start(ap, fmt);
    vprintf(fmt, ap);
    va_end(ap);
}
#endif

struct indent_measure {
  string name;
  long long starttime = getVa();

  ~indent_measure() {
    iprintf("done in: %s\n", timeshow(getVa() - starttime).c_str());
    fflush(stdout);
    indent -= 2;
    }

  indent_measure(string n) : name(n) { 
    iprintf("%s\n", name.c_str());
    fflush(stdout);
    indent += 2; 
    }
  
  operator bool() { return true; }
  };

struct progressbar : indent_measure {
  static const int PBSIZE = 64;
  int step = -1, total, drawat = 0, count = -1;
  
  void operator ++ (int) { 
    step++;
    while(step >= drawat && total) {
      count++;
      drawat = (total * (count+1)) / PBSIZE;
      fprintf(stderr, "[");
      for(int k=0; k<count; k++) fprintf(stderr, "#");
      for(int k=count; k<64; k++) fprintf(stderr, "-");
      fprintf(stderr, "] %s [%s]\x1b[K\r", name.c_str(), timeshow(getVa() - starttime).c_str());
      fflush(stderr);
      }
    }
  
  ~progressbar() {
    fprintf(stderr, "\x1b[K");
    }

  progressbar(int t, string n) : indent_measure(n) { total = t; (*this)++; }
  };

}
